
def transition_points_precision_recall_fs(ground_truth_segments, detected_segments, tolerance):
    print "Ground truth segments: %s" % len(ground_truth_segments)
    print "Detected segments: %s" % len(detected_segments)

    correctly_detected_transition_points = 0

    for i in range(0, len(ground_truth_segments) - 1):
        ground_segment = ground_truth_segments[i]
        for detected_segment in detected_segments:
            if abs(detected_segment.start_seconds - ground_segment.end_seconds) <= tolerance:
                correctly_detected_transition_points += 1
                break

    print "Correctly detected transition points: %s" % correctly_detected_transition_points

    if len(detected_segments) > 0:
        precision = correctly_detected_transition_points / float(len(detected_segments)-1)
    else:
        precision = 0
    if len(ground_truth_segments) > 0:
        recall = correctly_detected_transition_points / float(len(ground_truth_segments)-1)
    else:
        recall = 0
    if precision + recall != 0:
        fs = (2 * precision * recall) / (precision + recall)
    else:
        fs = 0
    print "SEGMENT LEVEL EVALUATION => Precision: %s Recall: %s F1 Score: %s" % (precision, recall, fs)

    return precision, recall, fs




import numpy
from matplotlib.patches import Rectangle
from sac.model.audacity_label import AudacityLabel
from mpmath import e
import peakutils
import matplotlib
from sympy import Symbol

matplotlib.use('TKAgg')
import matplotlib.pyplot as plt

STEP = 1
REGION_MAX_SIZE = 35
PEAK_THRESHOLD = 0.1


def _boost(input_x):
    x = Symbol('x')
    expr = 0.01 + e ** (-x / 6 + 3)
    return expr.subs(x, input_x)


def calculate_region_sums_and_boundaries(start_position, sm, region_max_size=REGION_MAX_SIZE):
    """

    With the same starting point scan multiple growing regions(up to REGION_MAX_SIZE, calculate the sums and the
    region boundaries

    :param start_position:
    :param sm:
    :param region_max_size:
    :return:
    """

    region_sums = []
    region_boundaries = []

    for i in range(1, region_max_size):
        start_row = start_position
        end_row = start_position + i * STEP + 1

        start_col = start_position
        end_col = start_position + i * STEP + 1

        region_boundaries.append([start_row, end_row, start_col, end_col])

        # print "%s:%s, %s:%s" % (start_row, end_row, start_col, end_col)

        subarray = sm[start_row: end_row, start_col: end_col]
        subarray_sum = numpy.sum(subarray)
        region_sums.append(subarray_sum)

        if end_row == sm.shape[0]:
            break

    return region_sums, region_boundaries


def identify_homogeneous_region(region_sums, region_boundaries, peak_threshold = PEAK_THRESHOLD, debug=False):
    """

    :param region_sums:
    :param region_boundaries:
    :param peak_threshold:
    :param debug:
    :return: detected boundaries [start_row, end_row, start_col, end_col]
    """
    # calculate first and second derivatives
    diff1 = numpy.diff(region_sums)
    diff2 = numpy.diff(diff1)

    peaks = peakutils.indexes(diff2, thres=peak_threshold)

    if len(peaks) > 0:
        # print peaks
        # print [diff2[peak_position] for peak_position in peaks]
        peak_values = [_boost(peak_position) * diff2[peak_position] for peak_position in peaks]
        # print peak_values
        argmax = numpy.argmax(peak_values)
        boundaries = region_boundaries[peaks[argmax]]
    else:
        boundaries = region_boundaries[-1]

    if debug:
        plt.figure()
        plt.plot(region_sums)
        plt.plot(diff1)
        plt.plot(diff2)
        plt.show()

    segment = [boundaries[0], boundaries[1]]

    return boundaries, segment


def get_regions(timestamps, sm, draw=False, debug=False):

    if draw:
        fig = plt.figure()
        ax = fig.add_subplot(111)

    segments = []
    audacity_labels = []

    x = 0
    while x < sm.shape[0]:

        region_sums, region_boundaries = calculate_region_sums_and_boundaries(x, sm)
        b, segment = identify_homogeneous_region(region_sums, region_boundaries, debug)

        if draw:
            ax.add_patch(Rectangle((b[0], b[2]), b[1] - b[0], b[1] - b[0], fill=False, edgecolor='b'))

        segments.append(segment)
        # use as a new starting point the end of the detected segment
        x = b[1]

    if draw:
        ax.imshow(sm, cmap=plt.cm.gray)
        plt.show()

    segments[-1] = [segments[-1][0], segments[-1][1]-1]
    for segment in segments:
        audacity_labels.append(AudacityLabel(timestamps[segment[0]], timestamps[segment[1]], "-"))

    return audacity_labels



import csv
import shutil
import struct
import subprocess
import tempfile
import wave

import os
import re
import numpy as np
import itertools
from model.audacity_label import AudacityLabel


class Util(object):

    @staticmethod
    def split_data_based_on_annotation(X, Y, classes):
        data = {}
        for c in classes:
            data[c] = []

        for k, x in enumerate(X):
            sample_class = classes[Y[k]]
            data[sample_class].append(x.tolist())

        for c in classes:
            data[c] = np.array(data[c])

        return data


    @staticmethod
    def get_annotated_data_x_y(timestamps, data, lbls):

        timestamps = np.array(timestamps)

        X = None
        Y = []
        classes = []

        for i in range(0, len(timestamps)):

            for lbl in lbls:
                if lbl.start_seconds <= timestamps[i] <= lbl.end_seconds:
                    if X is None:
                        X = data[i, :]
                    else:
                        X = np.vstack((X, data[i, :]))
                    Y.append(lbl.label)

                    if lbl.label not in classes:
                        classes.append(lbl.label)

        return X, Y, classes

    @staticmethod
    def get_annotated_data(timestamps, data, lbls):
        timestamps = np.array(timestamps)

        annotated_data = {}

        for lbl in lbls:
            start = lbl.start_seconds
            end = lbl.end_seconds
            l_class = lbl.label

            indices = np.where(np.logical_and(timestamps >= start, timestamps <= end))[0]

            if not l_class in annotated_data:
                annotated_data[l_class] = data[indices, :]
            else:
                annotated_data[l_class] = np.vstack((annotated_data[l_class], data[indices, :]))

        return annotated_data

    @staticmethod
    def remove_dir(directory):
        if os.path.exists(directory):
            shutil.rmtree(directory)

    @staticmethod
    def make_dir(directory):
        if not os.path.exists(directory):
            os.makedirs(directory)

    @staticmethod
    def get_class_from_filename(filename):
        return filename.split("_")[-1].split(".")[0]

    @staticmethod
    def calculate_classes_percentages(classifications):
        percentages = {}

        for classification in classifications:
            if not classification in percentages:
                percentages[classification] = 0
            percentages[classification] += 1

        for k in percentages:
            percentages[k] = percentages[k] / float(len(classifications))

        return percentages

    @staticmethod
    def generate_labels_from_classifications(classifications, timestamps):
        """
        This is to generate continuous segments out of classified small windows
        :param classifications:
        :param timestamps:
        :return:
        """
        window_length = timestamps[1] - timestamps[0]
        combo_list = [(classifications[k], timestamps[k]) for k in range(0, len(classifications))]
        labels = []
        for k, g in itertools.groupby(combo_list, lambda x: x[0]):
            items = list(g)
            start_time = items[0][1]
            end_time = items[-1][1] + window_length
            label_class = items[0][0]
            labels.append(AudacityLabel(start_time, end_time, label_class))
        return labels

    @staticmethod
    def combine_adjacent_labels_of_the_same_class(input_labels):
        labels = []
        for k, g in itertools.groupby(input_labels, lambda x: x.label):
            items = list(g)
            start_time = items[0].start_seconds
            end_time = items[-1].end_seconds
            label_class = items[0].label
            labels.append(AudacityLabel(start_time, end_time, label_class))
        return labels

    @staticmethod
    def downsample(input_wav):
        downsampled_wav = tempfile.NamedTemporaryFile(suffix=".wav")
        subprocess.check_call(['/usr/local/bin/sox', input_wav, '-r', '1000', downsampled_wav.name])
        return downsampled_wav

    @staticmethod
    def get_sample_rate(input_wav):
        wave_file = wave.open(input_wav, 'r')
        return wave_file.getframerate()

    @staticmethod
    def read_wav_file(input_wav):
        wave_file = wave.open(input_wav, 'r')
        length = wave_file.getnframes()
        print(wave_file.getparams())

        channels = wave_file.getnchannels()
        sample_width = wave_file.getsampwidth()
        bytes_per_frame = channels * sample_width
        sample_rate = wave_file.getframerate()

        samples = []
        wave_file.rewind()

        for i in range(0, length):
            wave_data = wave_file.readframes(1)
            # print(len(waveData))
            # because we have 2 channels the waveData length is 4. so we get the first 2 bytes only (for one channel)
            data = struct.unpack("<h", wave_data[0:2])
            samples.append(data[0])

        timestamps = [c * 1.0 / sample_rate for c in range(0, len(samples))]

        print(len(wave_data))

        return timestamps, samples

    @staticmethod
    def read_downsampled_wav(input_wav):
        downsampled_wav = Util.downsample(input_wav)
        timestamps, samples = Util.read_wav_file(downsampled_wav.name)
        downsampled_wav.close()
        return timestamps, samples

    @staticmethod
    def parse_yaafe_header(csv_file):
        header = {}
        header_lines = []
        with open(csv_file, 'r') as csv_file:
            for i in range(0, 6):
                l = csv_file.readline()
                if l[0] == "%":
                    header_lines.append(l.strip())

        for line in header_lines:
            if 'samplerate' in line:
                header['samplerate'] = int(re.search('.*samplerate=(\d+).*', line).group(1))
            if 'yaafedefinition' in line:
                yaafedefinition = re.search('.*yaafedefinition=(.*)', line).group(1)

                yaafedefinition_groups = re.search('.*yaafedefinition=(.*)', line).group(1).split('>')
                base_conf = yaafedefinition_groups[0].strip()
                base_step_size = int(re.search('.*stepSize=(\d+).*', line).group(1))

                yaafedefinition_groups_without_derivate = []
                for group in yaafedefinition_groups:
                    if not "Derivate" in group:
                        yaafedefinition_groups_without_derivate.append(group)

                if len(yaafedefinition_groups_without_derivate) > 1:
                    for i in range(1, len(yaafedefinition_groups_without_derivate)):
                        step_nb_frames = int(re.search('.*StepNbFrames=(\d+).*', line).group(1))
                        base_step_size *= step_nb_frames
                header['effective_step_size'] = base_step_size

        return header

    @staticmethod
    def load_yaafe_csv(csv_file):
        header = Util.parse_yaafe_header(csv_file)
        features = []
        with open(csv_file, 'r') as csv_file:
            for line in csv_file:
                if line[0] != '%':
                    feature_vector_elements = line.split(',')
                    feature_vector_elements = [float(i) for i in feature_vector_elements]
                    features.append(feature_vector_elements)

        timestamps = [i * header['effective_step_size'] / float(header['samplerate']) for i in range(0, len(features))]

        return timestamps, np.array(features)

    @staticmethod
    def read_merged_features(audio_file, features):
        feature_list = []
        for feature in features:
            timestamps, feature_vectors = Util.load_yaafe_csv("%s.%s.csv" % (audio_file, feature))
            feature_list.append(feature_vectors)

        return timestamps, np.hstack((i for i in feature_list))

    @staticmethod
    def read_full_dataset(files, features, selected_features=[]):
        """

        :param files: a list of the audio files
        :param features: a list of the names of the audio features
        :param selected_features: a list of column positions
        :return:
        """

        data = []
        classes = []

        for file in files:
            class_id = file.split("_")[-1].split(".")[0]
            if not class_id in classes:
                classes.append(class_id)
            timestamps, x = Util.read_merged_features(file, features)
            y = [classes.index(class_id) for i in x]
            data.append({
                "x": x,
                "y": y
            })

        X = np.vstack((d["x"] for d in data))
        Y = np.hstack((d["y"] for d in data))

        if len(selected_features) == 0:
            return X, Y, classes
        else:
            return X[:, selected_features], Y, classes

    @staticmethod
    def read_audacity_labels(labels_file):
        audacity_labels = []
        with open(labels_file, 'rb') as csv_file:
            csv_reader = csv.reader(csv_file, delimiter='\t')
            for row in csv_reader:
                audacity_labels.append(AudacityLabel(row[0], row[1], row[2]))
        return audacity_labels

    @staticmethod
    def write_audacity_labels(audacity_labels, filename):
        """
        :param audacity_labels: list containing audacity label objects
        :return:
        """
        with open(filename, "wb") as f:
            for label in audacity_labels:
                f.write("%s\t%s\t%s\n" % (label.start_seconds, label.end_seconds, label.label))

    @staticmethod
    def read_feature_names_from_file(feature_plan):
        with open(feature_plan, "rb") as f:
            lines = f.readlines()

        features = []
        for line in lines:
            if line[0] != "#":
                features.append(line.split(":")[0])

        return features





from scipy.ndimage import uniform_filter
from scipy.signal import find_peaks_cwt
from scipy.spatial.distance import pdist, squareform
import numpy as np

import matplotlib
matplotlib.use('TKAgg')
import matplotlib.pyplot as plt

plt.rcParams['xtick.labelsize'] = 8


def calculate_similarity_matrix(feature_vectors, subarray_size=None):

    if subarray_size is None:
        subarray_size = feature_vectors.shape[0]

    size = feature_vectors.shape[0]
    sm = np.zeros((size, size))

    for i in range(0, size, subarray_size):
        subarray_distances = pdist(feature_vectors[i:i + subarray_size * 2], 'cosine')
        subarray_distances = squareform(subarray_distances)
        sm[i:i + subarray_distances.shape[0], i:i + subarray_distances.shape[0]] = subarray_distances
    return sm


def get_checkerboard_matrix(kernel_width):

    """
    example matrix for width = 2

    -1  -1    1   1
    -1  -1    1   1
     1   1   -1  -1
     1   1   -1  -1

    :param kernel_width:
    :return:
    """

    return np.vstack((
        np.hstack((
            -1 * np.ones((kernel_width, kernel_width)), np.ones((kernel_width, kernel_width))
        )),
        np.hstack((
            np.ones((kernel_width, kernel_width)), -1 * np.ones((kernel_width, kernel_width))
        ))
    ))


def checkerboard_matrix_filtering(similarity_matrix, kernel_width, peak_range):
    """
    Moving the checkerboard matrix over the main diagonal of the similarity matrix one sample at a time.

    :param similarity_matrix:
    :param peak_range: the number of samples in which the peak detection algorithms finds a peak (TODO:clarify)
    :param kernel_width: the size of one quarter of the checkerboard matrix
    :return: peaks and convolution values
    """

    checkerboard_matrix = get_checkerboard_matrix(kernel_width)

    # The values calculated in this step are starting from the 'kernel_width' position and ending
    # at length - kernel_width
    d = []
    for i in range(0, similarity_matrix.shape[0] - 2 * kernel_width):
        base = similarity_matrix[i:i + kernel_width * 2, i:i + kernel_width * 2]
        print base
        d.append(np.sum(np.multiply(base, checkerboard_matrix)))

    print d

    # The missing values from 0 to kernel_width are calculated here
    top_left_d = []
    for i in range(0, kernel_width):
        base = similarity_matrix[0:i + kernel_width, 0:i + kernel_width]
        top_left_d.append(np.sum(np.multiply(base, checkerboard_matrix[kernel_width - i:, kernel_width - i:])))

    # The missing kernel_width values at the bottom right are set to 0
    convolution_values = top_left_d + d + [0 for i in range(0, kernel_width)]

    peaks = find_peaks_cwt(convolution_values, np.arange(1, peak_range))
    peaks = [0] + peaks + [len(convolution_values)-1]
    return peaks, convolution_values


def calculate_segment_start_end_times_from_peak_positions(peaks, timestamps):
    window_in_seconds = get_window_in_seconds(timestamps)
    segments = []
    for i in range(0, len(peaks) - 1):
        segments.append([peaks[i] * window_in_seconds, peaks[i + 1] * window_in_seconds])

    return segments


def get_window_in_seconds(timestamps):
    return timestamps[1] - timestamps[0]


def draw_similarity_matrix(similarity_matrix, peaks, convolution_values, image_filename, timestamps):
    plt.figure()
    ax1 = plt.subplot2grid((9, 1), (0, 0), rowspan=8)
    plt.imshow(similarity_matrix, cmap=plt.cm.gray)

    ax2 = plt.subplot2grid((9, 1), (8, 0), sharex=ax1)
    plt.plot(convolution_values)
    plt.plot(peaks, [convolution_values[p] for p in peaks], 'x')

    ticks = [i for i in range(0, len(convolution_values), int(len(convolution_values) / 7))]

    plt.xticks(ticks, ["%.0f" % timestamps[t] for t in ticks])
    plt.savefig(image_filename, bbox_inches='tight')


def get_segments(feature_vectors, timestamps, kernel_width, peak_range, filter_width, save_image=False,
                 image_filename="similarity_matrix.png", subarray_size=None):

    sm = calculate_similarity_matrix(feature_vectors, subarray_size)

    sm = uniform_filter(sm, filter_width)

    peaks, convolution_values = checkerboard_matrix_filtering(sm, kernel_width, peak_range)

    if save_image:
        draw_similarity_matrix(sm, peaks, convolution_values, image_filename, timestamps)

    return calculate_segment_start_end_times_from_peak_positions(peaks, timestamps)






import os
from unittest import TestCase

from sac.model.audacity_label import AudacityLabel
from sac.util import Util
import numpy as np


class UtilTests(TestCase):
    def setUp(self):
        self.stats_csv = os.path.join(os.path.dirname(__file__), 'fixtures/example_yaafe_stats.csv')
        self.no_stats_csv = os.path.join(os.path.dirname(__file__), 'fixtures/example_yaafe_no_stats.csv')
        self.double_stats_csv = os.path.join(os.path.dirname(__file__), 'fixtures/example_yaafe_double_stats.csv')
        self.no_stats_derivate = os.path.join(os.path.dirname(__file__), 'fixtures/example_yaafe_no_stats_derivate.csv')
        self.stats_derivate = os.path.join(os.path.dirname(__file__), 'fixtures/example_yaafe_stats_derivate.csv')

    # # yaafe header parser
    def test_parse_yaafe_header_stats(self):
        header = Util.parse_yaafe_header(self.stats_csv)
        self.assertEqual(22050, header['samplerate'])
        self.assertEqual(15360, header['effective_step_size'])

    def test_parse_yaafe_header_no_stats(self):
        header = Util.parse_yaafe_header(self.no_stats_csv)
        self.assertEqual(22050, header['samplerate'])
        self.assertEqual(512, header['effective_step_size'])

    def test_parse_yaafe_header_double_stats(self):
        header = Util.parse_yaafe_header(self.double_stats_csv)
        self.assertEqual(22050, header['samplerate'])
        self.assertEqual(460800, header['effective_step_size'])

    def test_parse_yaafe_header_no_stats_derivate(self):
        header = Util.parse_yaafe_header(self.no_stats_derivate)
        self.assertEqual(22050, header['samplerate'])
        self.assertEqual(512, header['effective_step_size'])

    def test_parse_yaafe_header_stats_derivate(self):
        header = Util.parse_yaafe_header(self.stats_derivate)
        self.assertEqual(22050, header['samplerate'])
        self.assertEqual(15360, header['effective_step_size'])

    # # yaafe csv parser
    def test_load_yaafe_csv_no_stats(self):
        timestamps, features = Util.load_yaafe_csv(self.no_stats_csv)
        self.assertEqual((17, 2), features.shape)
        self.assertEqual(0.37151927437641724, timestamps[-1])

    def test_load_yaafe_csv_stats(self):
        timestamps, features = Util.load_yaafe_csv(self.stats_csv)
        self.assertEqual((17, 2), features.shape)
        self.assertEqual(11.145578231292516, timestamps[-1])

    def test_load_yaafe_csv_double_stats(self):
        timestamps, features = Util.load_yaafe_csv(self.double_stats_csv)
        self.assertEqual((17, 2), features.shape)
        self.assertEqual(334.3673469387755, timestamps[-1])

    # # merge classifications
    def test_generate_labels_from_classifications(self):
        classifications = [1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1]
        timestamps = [0.0, 0.6965986394557823, 1.3931972789115645, 2.089795918367347, 2.786394557823129,
                      3.4829931972789114, 4.179591836734694, 4.876190476190477, 5.572789115646258, 6.2693877551020405,
                      6.965986394557823, 7.662585034013605, 8.359183673469389, 9.05578231292517, 9.752380952380953,
                      10.448979591836734, 11.145578231292516, 11.842176870748299, 12.538775510204081,
                      13.235374149659863, 13.931972789115646, 14.628571428571428, 15.32517006802721, 16.021768707482995]

        labels = Util.generate_labels_from_classifications(classifications, timestamps)

        expected_labels = [AudacityLabel(0.0, 1.3931972789115645, 1),
                           AudacityLabel(1.3931972789115645, 9.752380952380953, 0),
                           AudacityLabel(9.752380952380953, 10.448979591836736, 1),
                           AudacityLabel(10.448979591836734, 11.145578231292516, 0),
                           AudacityLabel(11.145578231292516, 12.538775510204081, 1),
                           AudacityLabel(12.538775510204081, 13.235374149659863, 0),
                           AudacityLabel(13.235374149659863, 16.718367346938777, 1)]

        self.assertListEqual(expected_labels, labels)

    def test_combine_adjacent_labels_of_the_same_class(self):
        input_labels = [
            AudacityLabel(0, 10, "m"),
            AudacityLabel(10, 20, "m"),
            AudacityLabel(20, 21, "s"),
            AudacityLabel(21, 22, "s"),
            AudacityLabel(22, 23, "s"),
            AudacityLabel(23, 30, "m")
        ]
        expected_labels = [
            AudacityLabel(0, 20, "m"),
            AudacityLabel(20, 23, "s"),
            AudacityLabel(23, 30, "m"),
        ]

        actual_labels = Util.combine_adjacent_labels_of_the_same_class(input_labels)

        self.assertListEqual(expected_labels, actual_labels)

    def test_calculate_classes_percentages(self):
        classifications = [1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1]
        percentages = Util.calculate_classes_percentages(classifications)

        self.assertAlmostEqual(0.5833333333333334, percentages[0])
        self.assertAlmostEqual(0.4166666666666667, percentages[1])

    def test_get_annotated_data(self):
        timestamps = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
        data = np.random.rand(7, 10)
        labels = [
            AudacityLabel(1.2, 2.5, "m"),
            AudacityLabel(4.5, 6.0, "s")
        ]

        annotated_data = Util.get_annotated_data(timestamps, data, labels)
        self.assertTrue("m" in annotated_data)
        self.assertTrue("s" in annotated_data)
        self.assertTrue(data[2, :] in annotated_data["m"])
        self.assertTrue(data[5, :] in annotated_data["s"])
        self.assertTrue(data[6, :] in annotated_data["s"])

    def test_get_annotated_data_x_y(self):
        timestamps = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
        data = np.random.rand(7, 10)
        labels = [
            AudacityLabel(1.2, 2.5, "m"),
            AudacityLabel(4.5, 6.0, "s")
        ]

        x, y, classes = Util.get_annotated_data_x_y(timestamps, data, labels)

        self.assertEqual(3, x.shape[0])
        self.assertListEqual(["m", "s", "s"], y)
        self.assertListEqual(["m", "s"], classes)

    def test_split_data_based_on_annotation(self):
        X = np.array([
            [1,2,3,4],
            [2,3,4,5],
            [4,5,6,7]
        ])
        Y = [0,0,1]
        classes = ["music", "speech"]

        data = Util.split_data_based_on_annotation(X, Y, classes)

        self.assertEqual(data["music"].shape[0], 2)
        self.assertEqual(data["speech"].shape[0], 1)



from setuptools import setup

setup(
    entry_points={
        'console_scripts': [
            'sac = sac:main'
        ]
    },
    name='sac',
    packages=[
        'sac',
        'sac.cli',
        'sac.model',
        'sac.methods'
    ],
    version='0.5.1',
    description='Semantic Audio Companion',
    author='Nikolaos Tsipas',
    author_email='nicktgr15@yahoo.com',
    url='https://github.com/nicktgr15/sac',
    download_url='https://github.com/nicktgr15/sac/releases/tag/0.5.1',
    keywords=['semantic', 'audio', 'sox', 'similarity', 'matrix'],
    classifiers=[],
    install_requires=['matplotlib', 'numpy', 'scipy', 'scipy', 'sympy', 'peakutils']
)





class AudacityLabel(object):

    def __init__(self, start_seconds, end_seconds, label):
        self.label = label
        self.end_seconds = float(end_seconds)
        self.start_seconds = float(start_seconds)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __repr__(self):
        return str(self.__dict__)


import argparse
import sys

from sac.cli.wav_editor import WavEditor


class DefaultHelpParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)


def main():
    parser = DefaultHelpParser()
    subparsers = parser.add_subparsers(dest="command")
    subparsers.required = True

    # split-wav command parser
    split_wav_parser = subparsers.add_parser('split-wav', help='Splits a wave files in segments using '
                                                               'boundaries and class names provided in a labels file')
    split_wav_parser.add_argument('-l', '--labels', dest='labels', required=True,
                                  help='CSV labels file.')
    split_wav_parser.add_argument('-i', '--input-wav', dest='input_wav', required=True,
                                  help='The input wave file. Split wave files will use the same encoding.')
    split_wav_parser.add_argument('-o', '--output-dir', dest='output_dir', required=True, help='The output directory.')
    split_wav_parser.add_argument('-d', '--delimiter', dest='delimiter', help='Labels parser delimiter', default="\t")
    split_wav_parser.add_argument('-f', '--labels-format', dest='format', choices=['f1', 'f2'], required=True,
                                  help='The labels format. f1: start,end,class f2: start,duration,class')

    # concat-wav command parser
    concat_wav_parser = subparsers.add_parser('concat-wav', help='Combines wav files generated using the '
                                                                 'split-wav command.')
    concat_wav_parser.add_argument('-i', '--input-dir', dest='input_dir', required=True,
                                   help='The directory containing the split wav files.')
    concat_wav_parser.add_argument('-n', '--output-filename', dest='output_name', required=True,
                                   help='The name prefix of the generated files.')
    concat_wav_parser.add_argument('-o', '--output-dir', dest='output_dir', required=True,
                                   help='The output directory.')

    # split-concat-wav command parser
    split_concat_wav_parser = subparsers.add_parser('split-concat-wav', help='Splits and combines multiple wav files.')
    split_concat_wav_parser.add_argument('-i', '--files-labels-list', dest='files_labels_list_file', required=True,
                                         help='A list of comma separated <labels-file>,<wav-file> pairs that will be '
                                              'used as input. Relative to this file.')
    split_concat_wav_parser.add_argument('-o', '--output-dir', dest='output_dir', required=True,
                                         help='The output directory.')
    split_concat_wav_parser.add_argument('-d', '--delimiter', dest='delimiter', help='Labels parser delimiter',
                                         default="\t")
    split_concat_wav_parser.add_argument('-n', '--output-filename', dest='output_name', required=True,
                                         help='The name prefix of the generated files.')
    split_concat_wav_parser.add_argument('-f', '--labels-format', dest='format', choices=['f1', 'f2'], required=True,
                                         help='The labels format. f1: start,end,class f2: start,duration,class')

    args = parser.parse_args()

    if args.command == "split-wav":
        WavEditor.create_audio_segments(args.labels, args.input_wav, args.output_dir, False, args.delimiter,
                                        args.format)
    elif args.command == "concat-wav":
        WavEditor.combine_audio_segments(args.input_dir, args.output_name, args.output_dir, False)
    elif args.command == "split-concat-wav":
        WavEditor.split_concat(args.files_labels_list_file, args.output_name, args.output_dir, False, args.delimiter,
                               args.format)


import logging
import shutil
import subprocess
import csv
import tempfile
import uuid
import sys

import os
from sac.util import Util


class WavEditor(object):
    @staticmethod
    def create_audio_segment(start_time, end_time, input_wav, output_wav):
        duration = str(float(end_time) - float(start_time))
        subprocess.check_call(["sox", input_wav, output_wav, "trim", start_time, duration])

    @staticmethod
    def create_audio_segments(labels, input_wav, output_dir, clear_output_dir, delimiter, format):
        if clear_output_dir:
            Util.remove_dir(output_dir)
        Util.make_dir(output_dir)
        for k, row in enumerate(WavEditor.get_rows(labels, delimiter)):
            if format == "f1":
                start_time = row[0]
                end_time = row[1]
            elif format == "f2":
                start_time = row[0]
                end_time = str(float(row[0]) + float(row[1]))
            else:
                logging.error("not supported file format")
                sys.exit(1)

            label = row[2]
            filename = str(uuid.uuid4())
            WavEditor.create_audio_segment(start_time, end_time, input_wav,
                                           os.path.join(output_dir, "%s_%s.wav" % (filename, label)))


    @staticmethod
    def get_rows(filename, delimiter):
        rows = []
        with open(filename, 'r') as csv_file:
            reader = csv.reader(csv_file, delimiter=delimiter)
            for row in reader:
                if len(row) > 0:
                    rows.append(row)
        return rows

    @staticmethod
    def get_files_grouped_by_class(input_dir):
        input_dir = os.path.abspath(input_dir)
        files = os.listdir(input_dir)
        files_grouped_by_class = {}

        for file in files:
            file_class = Util.get_class_from_filename(file)
            if not file_class in files_grouped_by_class:
                files_grouped_by_class[file_class] = []
            files_grouped_by_class[file_class].append(os.path.join(input_dir, file))

        return files_grouped_by_class

    @staticmethod
    def combine_audio_segments(input_dir, output_name, output_dir, clear_output_dir):
        if clear_output_dir:
            Util.remove_dir(output_dir)
        Util.make_dir(output_dir)
        files_grouped_by_class = WavEditor.get_files_grouped_by_class(input_dir)

        for class_name in files_grouped_by_class:
            input_wavs = []
            for k, file in enumerate(files_grouped_by_class[class_name]):
                input_wavs.append(file)
            cmd = ['sox'] + input_wavs + [os.path.join(output_dir, '%s_%s.wav' % (output_name, class_name))]
            subprocess.check_call(cmd)

    @staticmethod
    def split_concat(files_labels_list_file, output_name, output_dir, clear_output_dir, delimiter, format):
        files_labels_list_file = os.path.abspath(files_labels_list_file)
        files_labels_list_dir = os.path.split(files_labels_list_file)[0]

        with open(files_labels_list_file, 'rb') as f:
            files_labels_list = f.readlines()

        temp_dir = tempfile.mkdtemp()

        for file_label_pair in files_labels_list:
            label_file, audio_file = file_label_pair.decode("utf-8").strip().split(",")
            label_file_abs = os.path.join(files_labels_list_dir, label_file)
            audio_file_abs = os.path.join(files_labels_list_dir, audio_file)
            print("%s :: %s" % (label_file_abs, audio_file_abs))
            WavEditor.create_audio_segments(label_file_abs, audio_file_abs, temp_dir, clear_output_dir, delimiter,
                                            format)

        WavEditor.combine_audio_segments(temp_dir, output_name, output_dir, clear_output_dir)

        # temp_dir.cleanup()
        shutil.rmtree(temp_dir)